@@ -1,3 +1,5 @@
+
+
 /*
  * Hedgewars, a free turn based strategy game
  * Copyright (c) 2007 Andrey Korotaev <unC0Rr@gmail.com>
@@ -19,8 +21,8 @@
 #ifndef SDLS_H
 #define SDLS_H
 
-#include <QStringList>
 
+#include <QStringList>
 #include "SDL_mixer.h"
 
 
@@ -29,14 +31,19 @@ class SDLInteraction : public QObject
 	Q_OBJECT
 
 private:
-	Mix_Music * music;
+	Mix_Music *music;
+	int musicInitialized;	
 
 public:
 	SDLInteraction();
 	~SDLInteraction();
 	QStringList getResolutions() const;
+	void addGameControllerKeys() const;
 	void StartMusic();
 	void StopMusic();
+	void SDLMusicInit();	
 };
 
+
 #endif
+
