@@ -25,6 +25,7 @@
 extern QString * cProtoVer;
 extern QString * cVersionString;
 extern QString * cDataDir;
+extern QString * cConfigDir;
 
 extern QDir * bindir;
 extern QDir * cfgdir;
@@ -34,6 +35,7 @@ extern QStringList * Themes;
 extern QStringList * mapList;
 
 extern QString * cDefaultAmmoStore;
+extern int cAmmoNumber;
 extern QList< QPair<QString, QString> > cDefaultAmmos;
 
 extern QColor * color1;
