@@ -55,6 +55,15 @@
 #include "ammoSchemeModel.h"
 #include "bgwidget.h"
 
+#ifdef __APPLE__
+#include "CocoaInitializer.h"
+#include "M3Panel.h"
+#ifdef SPARKLE_ENABLED
+#define SPARKLE_APPCAST_URL "http://www.hedgewars.org/download/appcast.xml"
+#include "SparkleAutoUpdater.h"
+#endif
+#endif
+
 // I started handing this down to each place it touches, but it was getting ridiculous
 // and this one flag does not warrant a static class
 bool frontendEffects = true;
@@ -68,13 +77,24 @@ HWForm::HWForm(QWidget *parent)
 	ui.setupUi(this);
 
 	CustomizePalettes();
-
+	
 	ui.pageOptions->CBResolution->addItems(sdli.getResolutions());
 
 	config = new GameUIConfig(this, cfgdir->absolutePath() + "/hedgewars.ini");
-	
+
 	namegen = new HWNamegen();
 
+#ifdef __APPLE__
+        panel = new M3Panel;
+#ifdef SPARKLE_ENABLED
+        AutoUpdater* updater;
+        CocoaInitializer initializer;
+        updater = new SparkleAutoUpdater(SPARKLE_APPCAST_URL);
+	if(updater && config->isAutoUpdateEnabled())
+            updater->checkForUpdates();
+#endif        
+#endif
+
 	UpdateTeamsLists();
 	UpdateWeapons();
 
@@ -120,7 +140,7 @@ HWForm::HWForm(QWidget *parent)
 	connect(ui.pageNet->BtnSpecifyServer, SIGNAL(clicked()), this, SLOT(NetConnect()));
 	connect(ui.pageNet->BtnNetSvrStart, SIGNAL(clicked()), this, SLOT(GoToNetServer()));
 	connect(ui.pageNet, SIGNAL(connectClicked(const QString &, quint16)), this, SLOT(NetConnectServer(const QString &, quint16)));
-	
+
 	connect(ui.pageNetServer->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 	connect(ui.pageNetServer->BtnStart, SIGNAL(clicked()), this, SLOT(NetStartServer()));
 
@@ -151,8 +171,6 @@ HWForm::HWForm(QWidget *parent)
 	connect(ui.pageTraining->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
 	connect(ui.pageSelectWeapon->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
-	connect(ui.pageSelectWeapon->BtnDefault, SIGNAL(clicked()), ui.pageSelectWeapon->pWeapons, SLOT(setDefault()));
-	connect(ui.pageSelectWeapon->BtnSave, SIGNAL(clicked()), ui.pageSelectWeapon->pWeapons, SLOT(save()));
 
 	connect(ui.pageSelectWeapon->BtnDelete, SIGNAL(clicked()),
 		ui.pageSelectWeapon->pWeapons, SLOT(deleteWeaponsName())); // executed first
@@ -223,7 +241,7 @@ void HWForm::UpdateWeapons()
 
 		for(int i = 0; i < names.size(); ++i)
 			(*it)->addItem(names[i], ui.pageSelectWeapon->pWeapons->getWeaponsString(names[i]));
-		
+
 		int pos = (*it)->findText("Default");
 		if (pos != -1) {
 			(*it)->setCurrentIndex(pos);
@@ -344,20 +362,20 @@ void HWForm::OnPageShown(quint8 id, quint8 lastid)
 	if (id == ID_PAGE_MULTIPLAYER || id == ID_PAGE_NETGAME) {
 		QStringList tmNames = config->GetTeamsList();
 		TeamSelWidget* curTeamSelWidget;
-		
+
 		if(id == ID_PAGE_MULTIPLAYER) {
 		  curTeamSelWidget = ui.pageMultiplayer->teamsSelect;
 		} else {
 		  curTeamSelWidget = ui.pageNetGame->pNetTeamsWidget;
 		}
-		
+
 		QList<HWTeam> teamsList;
 		for(QStringList::iterator it = tmNames.begin(); it != tmNames.end(); it++) {
 		  HWTeam team(*it);
 		  team.LoadFromFile();
 		  teamsList.push_back(team);
 		}
-		
+
 		if(lastid == ID_PAGE_SETUP) { // _TEAM
 		  if (editedTeam) {
 		    curTeamSelWidget->addTeam(*editedTeam);
@@ -389,14 +407,14 @@ void HWForm::GoBack()
 	quint8 curid = ui.Pages->currentIndex();
 	ui.Pages->setCurrentIndex(id);
 	OnPageShown(id, curid);
-	
+
 	if (id == ID_PAGE_CONNECTING)
 		GoBack();
 	if (id == ID_PAGE_NETSERVER)
 		GoBack();
 	if ((!hwnet) && (id == ID_PAGE_ROOMSLIST))
 		GoBack();
-	
+
 	if ((!hwnet) || (!hwnet->isInRoom()))
 		if (id == ID_PAGE_NETGAME || id == ID_PAGE_NETGAME)
 			GoBack();
@@ -414,9 +432,13 @@ void HWForm::btnExitPressed()
 
 void HWForm::btnExitClicked()
 {
-	if (eggTimer.elapsed() < 3000)
+	if (eggTimer.elapsed() < 3000){
+#ifdef __APPLE__
+                panel->showInstallController();
+#endif
 		close();
-	else
+	}
+        else
 	{
 		QPushButton * btn = findChild<QPushButton *>("imageButt");
 		if (btn)
@@ -430,13 +452,13 @@ void HWForm::IntermediateSetup()
 {
 	quint8 id=ui.Pages->currentIndex();
 	TeamSelWidget* curTeamSelWidget;
-	
+
 	if(id == ID_PAGE_MULTIPLAYER) {
 		curTeamSelWidget = ui.pageMultiplayer->teamsSelect;
 	} else {
 		curTeamSelWidget = ui.pageNetGame->pNetTeamsWidget;
 	}
-	
+
 	QList<HWTeam> teams = curTeamSelWidget->getDontPlayingTeams();
 	QStringList tmnames;
 	for(QList<HWTeam>::iterator it = teams.begin(); it != teams.end(); ++it) {
@@ -531,9 +553,9 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		delete hwnet;
 		hwnet=0;
 	}
-	
+
 	ui.pageRoomsList->chatWidget->clear();
-	
+
 	hwnet = new HWNewNet(config, ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget);
 
 	GoToPage(ID_PAGE_CONNECTING);
@@ -554,7 +576,7 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		ui.pageRoomsList, SLOT(setAdmin(bool)));
 	connect(hwnet, SIGNAL(adminAccess(bool)),
 		ui.pageRoomsList->chatWidget, SLOT(adminAccess(bool)));
-	
+
 	connect(hwnet, SIGNAL(serverMessage(const QString&)),
 		ui.pageRoomsList->chatWidget, SLOT(onServerMessage(const QString&)));
 
@@ -562,13 +584,17 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		hwnet, SLOT(CreateRoom(const QString&)));
 	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
 		hwnet, SLOT(JoinRoom(const QString&)));
-	connect(ui.pageRoomsList, SIGNAL(askForCreateRoom(const QString &)),
-		this, SLOT(NetGameMaster()));
-	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
-		this, SLOT(NetGameSlave()));
+//	connect(ui.pageRoomsList, SIGNAL(askForCreateRoom(const QString &)),
+//		this, SLOT(NetGameMaster()));
+//	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
+//		this, SLOT(NetGameSlave()));
 	connect(ui.pageRoomsList, SIGNAL(askForRoomList()),
 		hwnet, SLOT(askRoomsList()));
 
+// room status stuff
+	connect(hwnet, SIGNAL(roomMaster(bool)),
+		this, SLOT(NetGameChangeStatus(bool)));
+
 // net page stuff
 	connect(hwnet, SIGNAL(chatStringFromNet(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(onChatString(const QString&)));
@@ -632,10 +658,11 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 // admin stuff
 	connect(hwnet, SIGNAL(serverMessage(const QString&)), ui.pageAdmin, SLOT(serverMessage(const QString &)));
 	connect(ui.pageAdmin, SIGNAL(setServerMessage(const QString&)), hwnet, SLOT(newServerMessage(const QString &)));
+	connect(ui.pageAdmin->pbClearAccountsCache, SIGNAL(clicked()), hwnet, SLOT(clearAccountsCache()));
 
 // disconnect
 	connect(hwnet, SIGNAL(Disconnected()), this, SLOT(ForcedDisconnect()), Qt::QueuedConnection);
-	
+
 	hwnet->Connect(hostName, port, nick);
 }
 
@@ -710,7 +737,7 @@ void HWForm::ForcedDisconnect()
 		hwnet = 0;
 		QMessageBox::warning(this, QMessageBox::tr("Network"),
 				QMessageBox::tr("Connection to server is lost"));
-	
+
 	}
 	if (ui.Pages->currentIndex() != ID_PAGE_NET) GoBack();
 }
@@ -748,7 +775,7 @@ void HWForm::GameStateChanged(GameState gameState)
 	switch(gameState) {
 		case gsStarted: {
 			Music(false);
-			if (wBackground) wBackground->stopAnimation();	
+			if (wBackground) wBackground->stopAnimation();
 			GoToPage(ID_PAGE_INGAME);
 			ui.pageGameStats->clear();
 			if (pRegisterServer)
@@ -756,22 +783,25 @@ void HWForm::GameStateChanged(GameState gameState)
 				pRegisterServer->unregister();
 				pRegisterServer = 0;
 			}
+			//setVisible(false);
 			break;
 		}
 		case gsFinished: {
+			//setVisible(true);
 			GoBack();
 			Music(ui.pageOptions->CBEnableMusic->isChecked());
-			if (wBackground) wBackground->startAnimation();	
+			if (wBackground) wBackground->startAnimation();
 			GoToPage(ID_PAGE_GAMESTATS);
 			if (hwnet) hwnet->gameFinished();
 			break;
 		}
 		default: {
+			//setVisible(true);
 			quint8 id = ui.Pages->currentIndex();
 			if (id == ID_PAGE_INGAME) {
 				GoBack();
 				Music(ui.pageOptions->CBEnableMusic->isChecked());
-				if (wBackground) wBackground->startAnimation();	
+				if (wBackground) wBackground->startAnimation();
 				if (hwnet) hwnet->gameFinished();
 			}
 		};
@@ -831,7 +861,7 @@ void HWForm::StartTraining()
 {
 	CreateGame(0, 0, 0);
 
-	game->StartTraining();
+	game->StartTraining(ui.pageTraining->CBSelect->currentText());
 }
 
 void HWForm::CreateNetGame()
@@ -845,6 +875,7 @@ void HWForm::CreateNetGame()
 
 	connect(game, SIGNAL(SendNet(const QByteArray &)), hwnet, SLOT(SendNet(const QByteArray &)));
 	connect(game, SIGNAL(SendChat(const QString &)), hwnet, SLOT(chatLineToNet(const QString &)));
+	connect(game, SIGNAL(SendTeamMessage(const QString &)), hwnet, SLOT(SendTeamMessage(const QString &)));
 	connect(hwnet, SIGNAL(FromNet(const QByteArray &)), game, SLOT(FromNet(const QByteArray &)));
 	connect(hwnet, SIGNAL(chatStringFromNet(const QString &)), game, SLOT(FromNetChat(const QString &)));
 
@@ -865,13 +896,23 @@ void HWForm::Music(bool checked)
 		sdli.StopMusic();
 }
 
+void HWForm::NetGameChangeStatus(bool isMaster)
+{
+	if (isMaster)
+		NetGameMaster();
+	else
+		NetGameSlave();
+}
+
 void HWForm::NetGameMaster()
 {
 	ui.pageNetGame->setMasterMode(true);
 	ui.pageNetGame->restrictJoins->setChecked(false);
 	ui.pageNetGame->restrictTeamAdds->setChecked(false);
 	ui.pageNetGame->pGameCFG->GameSchemes->setModel(ammoSchemeModel);
-	
+	ui.pageNetGame->pGameCFG->setEnabled(true);
+	ui.pageNetGame->pNetTeamsWidget->setInteractivity(true);
+
 	if (hwnet)
 	{
 		// disconnect connections first to ensure their inexistance and not to connect twice
@@ -891,6 +932,9 @@ void HWForm::NetGameMaster()
 
 void HWForm::NetGameSlave()
 {
+	ui.pageNetGame->pGameCFG->setEnabled(false);
+	ui.pageNetGame->pNetTeamsWidget->setInteractivity(false);
+
 	if (hwnet)
 	{
 		NetAmmoSchemeModel * netAmmo = new NetAmmoSchemeModel(hwnet);
