@@ -37,7 +37,6 @@
 #include <QTableWidget>
 #include <QAction>
 #include <QMenu>
-#include <QSound>
 #include <QDataWidgetMapper>
 
 
@@ -61,8 +60,9 @@
 #include "misc.h"
 #include "togglebutton.h"
 #include "hwform.h"
+#include "SDLs.h"
 
-PageMain::PageMain(QWidget* parent) : 
+PageMain::PageMain(QWidget* parent) :
   AbstractPage(parent)
 {
     if(frontendEffects) setAttribute(Qt::WA_NoSystemBackground, true);
@@ -98,9 +98,10 @@ PageMain::PageMain(QWidget* parent) :
 	BtnExit = addButton(":/res/Exit.png", pageLayout, 4, 0, 1, 1, true);
 }
 
-PageEditTeam::PageEditTeam(QWidget* parent) :
+PageEditTeam::PageEditTeam(QWidget* parent, SDLInteraction * sdli) :
   AbstractPage(parent)
 {
+    mySdli = sdli;
 	QGridLayout * pageLayout = new QGridLayout(this);
 	QTabWidget * tbw = new QTabWidget(this);
 	QWidget * page1 = new QWidget(this);
@@ -111,7 +112,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 	BtnTeamDiscard = addButton(":/res/Exit.png", pageLayout, 1, 0, true);
 	BtnTeamSave = addButton(":/res/Save.png", pageLayout, 1, 2, true);;
 	BtnTeamSave->setStyleSheet("QPushButton{margin: 12px 0px 12px 0px;}");
-	
+
 	QHBoxLayout * page1Layout = new QHBoxLayout(page1);
 	page1Layout->setAlignment(Qt::AlignTop);
 	QGridLayout * page2Layout = new QGridLayout(page2);
@@ -123,14 +124,14 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 	page1Layout->addLayout(vbox1);
 	page1Layout->addLayout(vbox2);
 	page1Layout->addLayout(vbox3);
-	
+
 	GBoxHedgehogs = new QGroupBox(this);
 	GBoxHedgehogs->setTitle(QGroupBox::tr("Team Members"));
 	GBoxHedgehogs->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 	QGridLayout * GBHLayout = new QGridLayout(GBoxHedgehogs);
 
 	signalMapper = new QSignalMapper(this);
-	
+
 	HatsModel * hatsModel = new HatsModel(GBoxHedgehogs);
 	for(int i = 0; i < 8; i++)
 	{
@@ -146,7 +147,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 		HHNameEdit[i]->setMaxLength(64);
 		HHNameEdit[i]->setMinimumWidth(120);
 		GBHLayout->addWidget(HHNameEdit[i], i, 1);
-		
+
 		randButton[i] = addButton(":/res/dice.png", GBHLayout, i, 3, true);
 
 		connect(randButton[i], SIGNAL(clicked()), signalMapper, SLOT(map()));
@@ -167,7 +168,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 	TeamNameEdit->setMaxLength(64);
 	GBTLayout->addWidget(TeamNameEdit);
 	vbox2->addWidget(GBoxTeam);
-	
+
 	CBTeamLvl = new QComboBox(GBoxTeam);
 	CBTeamLvl->setIconSize(QSize(48, 48));
 	CBTeamLvl->addItem(QIcon(":/res/botlevels/0.png"), QComboBox::tr("Human"));
@@ -177,7 +178,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 				QString("%1 %2").arg(QComboBox::tr("Level")).arg(i)
 				);
 	GBTLayout->addWidget(CBTeamLvl);
-	
+
 	CBGrave = new QComboBox(GBoxTeam);
 	CBGrave->setMaxCount(65535);
 	CBGrave->setIconSize(QSize(32, 32));
@@ -221,7 +222,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 
 	connect(CBFort, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(CBFort_activated(const QString &)));
 	CBFort->addItems(tmpdir.entryList(QStringList("*L.png")).replaceInStrings(QRegExp("^(.*)L\\.png"), "\\1"));
-	
+
 	tmpdir.cd("../Graphics/Graves");
 	QStringList list = tmpdir.entryList(QStringList("*.png"));
 	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it )
@@ -242,39 +243,42 @@ PageEditTeam::PageEditTeam(QWidget* parent) :
 	BindsBox = new QToolBox(GBoxBinds);
 	BindsBox->setLineWidth(0);
 	GBBLayout->addWidget(BindsBox);
-	page_A = new QWidget(this);
-	BindsBox->addItem(page_A, QToolBox::tr("Actions"));
-	page_W = new QWidget(this);
-	BindsBox->addItem(page_W, QToolBox::tr("Weapons"));
-	page_WP = new QWidget(this);
-	BindsBox->addItem(page_WP, QToolBox::tr("Weapon properties"));
-	page_O = new QWidget(this);
-	BindsBox->addItem(page_O, QToolBox::tr("Other"));
 	page2Layout->addWidget(GBoxBinds, 0, 0);
 
-	QStringList binds;
-	for(int i = 0; strlen(sdlkeys[i][1]) > 0; i++)
-	{
-		binds << sdlkeys[i][1];
-	}
-
 	quint16 widind = 0, i = 0;
+	quint16 num = 0;
+	QWidget * curW = NULL;
+	QGridLayout * pagelayout = NULL;
+	QLabel* l = NULL;
 	while (i < BINDS_NUMBER) {
-		quint16 num = 0;
-		QWidget * curW = BindsBox->widget(widind);
-		QGridLayout * pagelayout = new QGridLayout(curW);
-		do {
-			LBind[i] = new QLabel(curW);
-			LBind[i]->setText(QApplication::translate("binds", cbinds[i].name));
-			LBind[i]->setAlignment(Qt::AlignRight);
-			pagelayout->addWidget(LBind[i], num, 0);
-			CBBind[i] = new QComboBox(curW);
-			CBBind[i]->addItems(binds);
-			pagelayout->addWidget(CBBind[i], num, 1);
-			num++;
-		} while (!cbinds[i++].chwidget);
-		pagelayout->addWidget(new QWidget(curW), num, 0, 1, 2);
-		widind++;
+		if(cbinds[i].category != NULL)
+		{
+			if(curW != NULL)
+			{
+				l = new QLabel(curW);
+				l->setText("");
+				pagelayout->addWidget(l, num++, 0, 1, 2);
+			}
+			curW = new QWidget(this);
+			BindsBox->addItem(curW, QApplication::translate("binds (categories)", cbinds[i].category));
+			pagelayout = new QGridLayout(curW);
+			num = 0;
+		}
+		if(cbinds[i].description != NULL)
+		{
+			l = new QLabel(curW);
+			l->setText((num > 0 ? QString("\n") : QString("")) + QApplication::translate("binds (descriptions)", cbinds[i].description));
+			pagelayout->addWidget(l, num++, 0, 1, 2);
+		}
+
+		l = new QLabel(curW);
+		l->setText(QApplication::translate("binds", cbinds[i].name));
+		l->setAlignment(Qt::AlignRight);
+		pagelayout->addWidget(l, num, 0);
+		CBBind[i] = new QComboBox(curW);
+		for(int j = 0; sdlkeys[j][1][0] != '\0'; j++)
+			CBBind[i]->addItem(QApplication::translate("binds (keys)", sdlkeys[j][1]).contains(": ") ? QApplication::translate("binds (keys)", sdlkeys[j][1]) : QApplication::translate("binds (keys)", "Keyboard") + QString(": ") + QApplication::translate("binds (keys)", sdlkeys[j][1]), sdlkeys[j][0]);
+		pagelayout->addWidget(CBBind[i++], num++, 1);
 	}
 }
 
@@ -286,13 +290,18 @@ void PageEditTeam::CBFort_activated(const QString & fortname)
 
 void PageEditTeam::testSound()
 {
+	Mix_Music *sound;
 	QDir tmpdir;
+	mySdli->SDLMusicInit();
+	
 	tmpdir.cd(datadir->absolutePath());
 	tmpdir.cd("Sounds/voices");
 	tmpdir.cd(CBVoicepack->currentText());
-	QStringList list = tmpdir.entryList(QStringList() << "*.wav", QDir::Files);
-	if (list.size())
-		QSound::play(tmpdir.absolutePath() + "/" + list[rand() % list.size()]);
+	QStringList list = tmpdir.entryList(QStringList() << "Illgetyou.ogg" << "Incoming.ogg" << "Stupid.ogg" << "Coward.ogg" << "Firstblood.ogg", QDir::Files);
+	if (list.size()) {
+		sound = Mix_LoadMUS(QString(tmpdir.absolutePath() + "/" + list[rand() % list.size()]).toLocal8Bit().constData());
+		Mix_PlayMusic(sound, 0);
+	}
 }
 
 PageMultiplayer::PageMultiplayer(QWidget* parent) :
@@ -333,128 +342,133 @@ PageOptions::PageOptions(QWidget* parent) :
 	gbTBLayout->setMargin(0);
 	gbTBLayout->setSpacing(0);
 	{
-		teamsBox = new IconedGroupBox(this);
-        //teamsBox->setAttribute(Qt::WA_PaintOnScreen, true);
-		teamsBox->setIcon(QIcon(":/res/teamicon.png"));
-		teamsBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-		teamsBox->setTitle(QGroupBox::tr("Teams"));
+            teamsBox = new IconedGroupBox(this);
+            //teamsBox->setAttribute(Qt::WA_PaintOnScreen, true);
+            teamsBox->setIcon(QIcon(":/res/teamicon.png"));
+            teamsBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+            teamsBox->setTitle(QGroupBox::tr("Teams"));
 
-		QVBoxLayout * GBTlayout = new QVBoxLayout(teamsBox);
+            QVBoxLayout * GBTlayout = new QVBoxLayout(teamsBox);
 
-		CBTeamName = new QComboBox(teamsBox);
-		GBTlayout->addWidget(CBTeamName);
+            CBTeamName = new QComboBox(teamsBox);
+            GBTlayout->addWidget(CBTeamName);
 
-		QHBoxLayout * layout1 = new QHBoxLayout;
-		GBTlayout->addLayout(layout1);
-		BtnNewTeam = addButton(tr("New team"), layout1, 0);
-		BtnEditTeam = addButton(tr("Edit team"), layout1, 1);
-		layout1->setStretchFactor(BtnNewTeam, 100);
-		layout1->setStretchFactor(BtnEditTeam, 100);
-		
-		QHBoxLayout * layout2 = new QHBoxLayout;
-		GBTlayout->addLayout(layout2);
-		
-		labelNN = new QLabel(teamsBox);
-		labelNN->setText(QLabel::tr("Net nick"));
-		layout2->addWidget(labelNN);
+            QHBoxLayout * layout1 = new QHBoxLayout;
+            GBTlayout->addLayout(layout1);
+            BtnNewTeam = addButton(tr("New team"), layout1, 0);
+            BtnEditTeam = addButton(tr("Edit team"), layout1, 1);
+            layout1->setStretchFactor(BtnNewTeam, 100);
+            layout1->setStretchFactor(BtnEditTeam, 100);
 
-		editNetNick = new QLineEdit(teamsBox);
-		editNetNick->setMaxLength(20);
-		editNetNick->setText(QLineEdit::tr("unnamed"));
-		layout2->addWidget(editNetNick);
-		
-		gbTBLayout->addWidget(teamsBox, 0, 0);
+            QHBoxLayout * layout2 = new QHBoxLayout;
+            GBTlayout->addLayout(layout2);
+
+            labelNN = new QLabel(teamsBox);
+            labelNN->setText(QLabel::tr("Net nick"));
+            layout2->addWidget(labelNN);
+
+            editNetNick = new QLineEdit(teamsBox);
+            editNetNick->setMaxLength(20);
+            editNetNick->setText(QLineEdit::tr("unnamed"));
+            layout2->addWidget(editNetNick);
+
+            gbTBLayout->addWidget(teamsBox, 0, 0);
 	}
 
-	{
-		IconedGroupBox* groupWeapons = new IconedGroupBox(this);
-        groupWeapons->setContentTopPadding(0);
-		groupWeapons->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-		groupWeapons->setIcon(QIcon(":/res/weaponsicon.png"));
-		//groupWeapons->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-		groupWeapons->setTitle(QGroupBox::tr("Weapons"));
-		QGridLayout * WeaponsLayout = new QGridLayout(groupWeapons);
+        {
+            IconedGroupBox* groupWeapons = new IconedGroupBox(this);
+            groupWeapons->setContentTopPadding(0);
+            groupWeapons->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            groupWeapons->setIcon(QIcon(":/res/weaponsicon.png"));
+            //groupWeapons->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+            groupWeapons->setTitle(QGroupBox::tr("Weapons"));
+            QGridLayout * WeaponsLayout = new QGridLayout(groupWeapons);
 
-		WeaponsButt = addButton(tr("Weapons set"), WeaponsLayout, 1, 0);
-		WeaponsName = new QComboBox(this);
-		WeaponsLayout->addWidget(WeaponsName, 0, 0, 1, 2);
-		WeaponEdit = addButton(tr("Edit"), WeaponsLayout, 1, 1);
-		gbTBLayout->addWidget(groupWeapons, 1, 0);
-	}
+            WeaponsButt = addButton(tr("Weapons set"), WeaponsLayout, 1, 0);
+            WeaponsName = new QComboBox(this);
+            WeaponsLayout->addWidget(WeaponsName, 0, 0, 1, 2);
+            WeaponEdit = addButton(tr("Edit"), WeaponsLayout, 1, 1);
+            gbTBLayout->addWidget(groupWeapons, 1, 0);
+        }
 
-	{
-		AGGroupBox = new IconedGroupBox(this);
-		AGGroupBox->setIcon(QIcon(":/res/graphicsicon.png"));
-		AGGroupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-		AGGroupBox->setTitle(QGroupBox::tr("Audio/Graphic options"));
+        {
+            AGGroupBox = new IconedGroupBox(this);
+            AGGroupBox->setIcon(QIcon(":/res/graphicsicon.png"));
+            AGGroupBox->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+            AGGroupBox->setTitle(QGroupBox::tr("Audio/Graphic options"));
 
-		QVBoxLayout * GBAlayout = new QVBoxLayout(AGGroupBox);
-		QHBoxLayout * GBAreslayout = new QHBoxLayout(0);
-		QLabel * resolution = new QLabel(AGGroupBox);
-		resolution->setText(QLabel::tr("Resolution"));
-		GBAreslayout->addWidget(resolution);
+            QVBoxLayout * GBAlayout = new QVBoxLayout(AGGroupBox);
+            QHBoxLayout * GBAreslayout = new QHBoxLayout(0);
 
-		CBResolution = new QComboBox(AGGroupBox);
-		GBAreslayout->addWidget(CBResolution);
-		GBAlayout->addLayout(GBAreslayout);
+            CBFrontendFullscreen = new QCheckBox(AGGroupBox);
+            CBFrontendFullscreen->setText(QCheckBox::tr("Frontend fullscreen"));
+            GBAlayout->addWidget(CBFrontendFullscreen);
 
-		QHBoxLayout * GBAfpslayout = new QHBoxLayout(0);
-		QLabel * maxfps = new QLabel(AGGroupBox);
-		maxfps->setText(QLabel::tr("FPS limit"));
-		GBAfpslayout->addWidget(maxfps);
-		GBAlayout->addLayout(GBAfpslayout);
+            CBFrontendEffects = new QCheckBox(AGGroupBox);
+            CBFrontendEffects->setText(QCheckBox::tr("Frontend effects (requires restart)"));
+            GBAlayout->addWidget(CBFrontendEffects);
 
-		CBReduceQuality = new QCheckBox(AGGroupBox);
-		CBReduceQuality->setText(QCheckBox::tr("Reduce Quality"));
-		GBAlayout->addWidget(CBReduceQuality);
+            QLabel * resolution = new QLabel(AGGroupBox);
+            resolution->setText(QLabel::tr("Resolution"));
+            GBAreslayout->addWidget(resolution);
 
-		CBFrontendEffects = new QCheckBox(AGGroupBox);
-		CBFrontendEffects->setText(QCheckBox::tr("Frontend Effects (Requires Restart)"));
-		GBAlayout->addWidget(CBFrontendEffects);
+            CBResolution = new QComboBox(AGGroupBox);
+            GBAreslayout->addWidget(CBResolution);
+            GBAlayout->addLayout(GBAreslayout);
 
-		CBFullscreen = new QCheckBox(AGGroupBox);
-		CBFullscreen->setText(QCheckBox::tr("Fullscreen"));
-		GBAlayout->addWidget(CBFullscreen);
+            QHBoxLayout * GBAfpslayout = new QHBoxLayout(0);
+            QLabel * maxfps = new QLabel(AGGroupBox);
+            maxfps->setText(QLabel::tr("FPS limit"));
+            GBAfpslayout->addWidget(maxfps);
+            GBAlayout->addLayout(GBAfpslayout);
 
-		CBFrontendFullscreen = new QCheckBox(AGGroupBox);
-		CBFrontendFullscreen->setText(QCheckBox::tr("Frontend fullscreen"));
-		GBAlayout->addWidget(CBFrontendFullscreen);
+            CBFullscreen = new QCheckBox(AGGroupBox);
+            CBFullscreen->setText(QCheckBox::tr("Fullscreen"));
+            GBAlayout->addWidget(CBFullscreen);
 
-		CBEnableSound = new QCheckBox(AGGroupBox);
-		CBEnableSound->setText(QCheckBox::tr("Enable sound"));
-		GBAlayout->addWidget(CBEnableSound);
+            CBReduceQuality = new QCheckBox(AGGroupBox);
+            CBReduceQuality->setText(QCheckBox::tr("Reduced quality"));
+            GBAlayout->addWidget(CBReduceQuality);
 
-		CBEnableMusic = new QCheckBox(AGGroupBox);
-		CBEnableMusic->setText(QCheckBox::tr("Enable music"));
-		GBAlayout->addWidget(CBEnableMusic);
+            CBEnableSound = new QCheckBox(AGGroupBox);
+            CBEnableSound->setText(QCheckBox::tr("Enable sound"));
+            GBAlayout->addWidget(CBEnableSound);
+            CBEnableMusic = new QCheckBox(AGGroupBox);
+            CBEnableMusic->setText(QCheckBox::tr("Enable music"));
+            GBAlayout->addWidget(CBEnableMusic);
 
-		QHBoxLayout * GBAvollayout = new QHBoxLayout(0);
-		QLabel * vol = new QLabel(AGGroupBox);
-		vol->setText(QLabel::tr("Initial sound volume"));
-		GBAvollayout->addWidget(vol);
-		GBAlayout->addLayout(GBAvollayout);
-		volumeBox = new QSpinBox(AGGroupBox);
-		volumeBox->setRange(0, 100);
-		volumeBox->setSingleStep(5);
-		GBAvollayout->addWidget(volumeBox);
-		
+            QHBoxLayout * GBAvollayout = new QHBoxLayout(0);
+            QLabel * vol = new QLabel(AGGroupBox);
+            vol->setText(QLabel::tr("Initial sound volume"));
+            GBAvollayout->addWidget(vol);
+            GBAlayout->addLayout(GBAvollayout);
+            volumeBox = new QSpinBox(AGGroupBox);
+            volumeBox->setRange(0, 100);
+            volumeBox->setSingleStep(5);
+            GBAvollayout->addWidget(volumeBox);
 
-		CBShowFPS = new QCheckBox(AGGroupBox);
-		CBShowFPS->setText(QCheckBox::tr("Show FPS"));
-		GBAlayout->addWidget(CBShowFPS);
+            CBShowFPS = new QCheckBox(AGGroupBox);
+            CBShowFPS->setText(QCheckBox::tr("Show FPS"));
+            GBAlayout->addWidget(CBShowFPS);
 
-		CBAltDamage = new QCheckBox(AGGroupBox);
-		CBAltDamage->setText(QCheckBox::tr("Alternative damage show"));
-		GBAlayout->addWidget(CBAltDamage);
+            CBAltDamage = new QCheckBox(AGGroupBox);
+            CBAltDamage->setText(QCheckBox::tr("Alternative damage show"));
+            GBAlayout->addWidget(CBAltDamage);
 
-		CBNameWithDate = new QCheckBox(AGGroupBox);
-		CBNameWithDate->setText(QCheckBox::tr("Append date and time to record file name"));
-		GBAlayout->addWidget(CBNameWithDate);
+            CBNameWithDate = new QCheckBox(AGGroupBox);
+            CBNameWithDate->setText(QCheckBox::tr("Append date and time to record file name"));
+            GBAlayout->addWidget(CBNameWithDate);
 
-		fpsedit = new FPSEdit(AGGroupBox);
-		GBAfpslayout->addWidget(fpsedit);
-		gbTBLayout->addWidget(AGGroupBox, 0, 1, 2, 1);
-	}
+#ifdef SPARKLE_ENABLED
+            CBAutoUpdate = new QCheckBox(AGGroupBox);
+            CBAutoUpdate->setText(QCheckBox::tr("Check for updates at startup"));
+            GBAlayout->addWidget(CBAutoUpdate);
+#endif
+
+            fpsedit = new FPSEdit(AGGroupBox);
+            GBAfpslayout->addWidget(fpsedit);
+            gbTBLayout->addWidget(AGGroupBox, 0, 1, 2, 1);
+        }
 
 	BtnSaveOptions = addButton(":/res/Save.png", pageLayout, 2, 2, true);
 	BtnSaveOptions->setStyleSheet("QPushButton{margin: 12px 0px 12px 0px;}");
@@ -477,7 +491,7 @@ PageNet::PageNet(QWidget* parent) : AbstractPage(parent)
 	pageLayout->addWidget(BtnNetSvrStart, 4, 2);
 
 	BtnBack = addButton(":/res/Exit.png", pageLayout, 4, 0, true);
-	
+
 	ConnGroupBox = new QGroupBox(this);
 	ConnGroupBox->setTitle(QGroupBox::tr("Net game"));
 	pageLayout->addWidget(ConnGroupBox, 2, 0, 1, 3);
@@ -622,7 +636,7 @@ PageNetGame::PageNetGame(QWidget* parent) : AbstractPage(parent)
 	pageLayout->addLayout(bottomLayout, 3, 0, 1, 2);
 
 	BtnBack = addButton(":/res/Exit.png", bottomLayout, 0, true);
-	
+
 	BtnGo = new QPushButton(this);
 	BtnGo->setToolTip(QPushButton::tr("Ready"));
 	BtnGo->setIcon(QIcon(":/res/lightbulb_off.png"));
@@ -641,7 +655,7 @@ PageNetGame::PageNetGame(QWidget* parent) : AbstractPage(parent)
 	//menu->addAction(startGame);
 	menu->addAction(restrictJoins);
 	menu->addAction(restrictTeamAdds);
-	
+
 	BtnMaster->setMenu(menu);
 
 	BtnStart = addButton(QAction::tr("Start"), bottomLayout, 3);
@@ -696,11 +710,11 @@ PageSinglePlayer::PageSinglePlayer(QWidget* parent) : AbstractPage(parent)
 	BtnMultiplayer = addButton(":/res/Multiplayer.png", topLine, 1, true);
 	BtnMultiplayer->setToolTip(tr("Multiplayer (play a hotseat game against your friends, or AI teams)"));
 	topLine->addStretch();
-	
+
 
 	BtnTrainPage = addButton(":/res/Trainings.png", middleLine, 0, true);
 	BtnTrainPage->setToolTip(tr("Training Mode (Practice your skills in a range of training missions). IN DEVELOPMENT"));
-	
+
 	BtnBack = addButton(":/res/Exit.png", bottomLine, 0, true);
 	bottomLine->addStretch();
 
@@ -718,6 +732,16 @@ PageTraining::PageTraining(QWidget* parent) : AbstractPage(parent)
 	pageLayout->setColumnStretch(1, 2);
 	pageLayout->setColumnStretch(2, 1);
 
+	CBSelect = new QComboBox(this);
+
+	QDir tmpdir;
+	tmpdir.cd(datadir->absolutePath());
+	tmpdir.cd("Trainings");
+	tmpdir.setFilter(QDir::Files);
+	CBSelect->addItems(tmpdir.entryList(QStringList("*_*.txt")).replaceInStrings(QRegExp("^(.*)\\.txt"), "\\1"));
+
+	pageLayout->addWidget(CBSelect, 1, 1);
+	
 	BtnStartTrain = new QPushButton(this);
 	BtnStartTrain->setFont(*font14);
 	BtnStartTrain->setText(QPushButton::tr("Go!"));
@@ -730,15 +754,18 @@ PageSelectWeapon::PageSelectWeapon(QWidget* parent) :
   AbstractPage(parent)
 {
 	QGridLayout * pageLayout = new QGridLayout(this);
-	
-	pWeapons = new SelWeaponWidget(cDefaultAmmoStore->size(), this);
+
+	pWeapons = new SelWeaponWidget(cAmmoNumber, this);
 	pageLayout->addWidget(pWeapons, 0, 0, 1, 4);
 
 	BtnBack = addButton(":/res/Exit.png", pageLayout, 1, 0, true);
 	BtnDefault = addButton(tr("Default"), pageLayout, 1, 1);
 	BtnDelete = addButton(tr("Delete"), pageLayout, 1, 2);
 	BtnSave = addButton(":/res/Save.png", pageLayout, 1, 3, true);
-	BtnSave->setStyleSheet("QPushButton{margin: 12px 0px 12px 0px;}");	
+	BtnSave->setStyleSheet("QPushButton{margin: 12px 0px 12px 0px;}");
+
+	connect(BtnDefault, SIGNAL(clicked()), pWeapons, SLOT(setDefault()));
+	connect(BtnSave, SIGNAL(clicked()), pWeapons, SLOT(save()));
 }
 
 PageInGame::PageInGame(QWidget* parent) :
@@ -753,19 +780,23 @@ PageRoomsList::PageRoomsList(QWidget* parent) :
 {
 	QGridLayout * pageLayout = new QGridLayout(this);
 
+	QHBoxLayout * newRoomLayout = new QHBoxLayout();
+	QLabel * roomNameLabel = new QLabel(this);
+	roomNameLabel->setText(tr("Room Name:"));
 	roomName = new QLineEdit(this);
 	roomName->setMaxLength(60);
-	pageLayout->addWidget(roomName, 0, 0);
-	
+	newRoomLayout->addWidget(roomNameLabel);
+	newRoomLayout->addWidget(roomName);
+	pageLayout->addLayout(newRoomLayout, 0, 0);
+
 	roomsList = new QTableWidget(this);
-	roomsList->setColumnCount(3);
 	roomsList->setSelectionBehavior(QAbstractItemView::SelectRows);
 	roomsList->verticalHeader()->setVisible(false);
 	roomsList->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
 	roomsList->setAlternatingRowColors(true);
 	pageLayout->addWidget(roomsList, 1, 0, 3, 1);
 	pageLayout->setRowStretch(2, 100);
-	
+
 	chatWidget = new HWChatWidget(this);
 	pageLayout->addWidget(chatWidget, 4, 0, 1, 2);
 	pageLayout->setRowStretch(4, 350);
@@ -773,7 +804,7 @@ PageRoomsList::PageRoomsList(QWidget* parent) :
 	BtnCreate = addButton(tr("Create"), pageLayout, 0, 1);
 	BtnJoin = addButton(tr("Join"), pageLayout, 1, 1);
 	BtnRefresh = addButton(tr("Refresh"), pageLayout, 3, 1);
-	
+
 	BtnBack = addButton(":/res/Exit.png", pageLayout, 5, 0, true);
 	BtnAdmin = addButton(tr("Admin features"), pageLayout, 5, 1);
 
@@ -791,29 +822,91 @@ void PageRoomsList::setAdmin(bool flag)
 void PageRoomsList::setRoomsList(const QStringList & list)
 {
 	roomsList->clear();
+	roomsList->setColumnCount(7);
 	roomsList->setHorizontalHeaderLabels(
 			QStringList() <<
-			QTableWidget::tr("Room name") <<
-			QTableWidget::tr("Players number") <<
-			QTableWidget::tr("Round in progress")
+			QTableWidget::tr("Room Name") <<
+			QTableWidget::tr("C") <<
+			QTableWidget::tr("T") <<
+			QTableWidget::tr("Owner") <<
+			QTableWidget::tr("Map") <<
+			QTableWidget::tr("Rules") <<
+			QTableWidget::tr("Weapons")
 			);
 
-   roomsList->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
-   roomsList->horizontalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
-   roomsList->horizontalHeader()->setResizeMode(2, QHeaderView::ResizeToContents);
+	// set minimum sizes
+	roomsList->horizontalHeader()->resizeSection(0, 200);
+	roomsList->horizontalHeader()->resizeSection(1, 50);
+	roomsList->horizontalHeader()->resizeSection(2, 50);
+	roomsList->horizontalHeader()->resizeSection(3, 100);
+	roomsList->horizontalHeader()->resizeSection(4, 100);
+	roomsList->horizontalHeader()->resizeSection(5, 100);
+	roomsList->horizontalHeader()->resizeSection(6, 100);
 
-	
-	if (list.size() % 3)
+	// set resize modes
+	roomsList->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
+
+	if (list.size() % 8)
 		return;
 
-	roomsList->setRowCount(list.size() / 3);
-	for(int i = 0; i < list.size(); i += 3)
-		for(int t = 0; t < 3; t++)
+	roomsList->setRowCount(list.size() / 8);
+	for(int i = 0, r = 0; i < list.size(); i += 8, r++)
+	{
+		QTableWidgetItem * item;
+		item = new QTableWidgetItem(list[i + 1]); // room name
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		
+		// pick appropriate room icon and tooltip (game in progress yes/no; later maybe locked rooms etc.)
+		if(list[i].compare("True"))
 		{
-			QTableWidgetItem * item = new QTableWidgetItem(list[i + t]);
-			item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
-			roomsList->setItem(i / 3, t, item);
+			item->setIcon(QIcon(":/res/iconTime.png"));// game is in lobby
+			item->setToolTip(tr("This game is in lobby.\nYou may join and start playing once the game starts."));
 		}
+		else
+		{
+			item->setIcon(QIcon(":/res/iconDamage.png"));// game has started
+			item->setToolTip(tr("This game is in progress.\nYou may join and spectate now but you'll have to wait for the game to end to start playing."));
+		}
+
+		roomsList->setItem(r, 0, item);
+
+		item = new QTableWidgetItem(list[i + 2]); // number of clients
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setTextAlignment(Qt::AlignCenter);
+		item->setToolTip(tr("There are %1 clients connected to this room.", "", list[i + 2].toInt()).arg(list[i + 2]));
+		roomsList->setItem(r, 1, item);
+
+		item = new QTableWidgetItem(list[i + 3]); // number of teams
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setTextAlignment(Qt::AlignCenter);
+		item->setToolTip(tr("There are %1 teams participating in this room.", "", list[i + 3].toInt()).arg(list[i + 3]));
+		roomsList->setItem(r, 2, item);
+
+		item = new QTableWidgetItem(list[i + 4]); // name of host
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setToolTip(tr("%1 is the host. He may adjust settings and start the game.").arg(list[i + 4]));
+		roomsList->setItem(r, 3, item);
+
+		if(list[i + 5].compare("+rnd+"))
+			item = new QTableWidgetItem(list[i + 5]); // selected map
+		else
+			item = new QTableWidgetItem(tr("Random Map")); // selected map (is randomized)
+		
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setToolTip(tr("Games may be played on precreated or randomized maps."));
+		roomsList->setItem(r, 4, item);
+
+		item = new QTableWidgetItem(list[i + 6]); // selected game scheme
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setToolTip(tr("The Game Scheme defines general options and preferences like Round Time, Sudden Death or Vampirism."));
+		roomsList->setItem(r, 5, item);
+
+		item = new QTableWidgetItem(list[i + 7]); // selected weapon scheme
+		item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+		item->setToolTip(tr("The Weapon Scheme defines available weapons and their ammunition count."));
+		roomsList->setItem(r, 6, item);
+
+	}
 	//roomsList->resizeColumnsToContents();
 }
 
@@ -863,7 +956,7 @@ PageScheme::PageScheme(QWidget* parent) :
 {
 	QGridLayout * pageLayout = new QGridLayout(this);
 	QGroupBox * gb = new QGroupBox(this);
-	
+
 	QGridLayout * gl = new QGridLayout();
 	gb->setLayout(gl);
 	QSizePolicy sp;
@@ -871,7 +964,7 @@ PageScheme::PageScheme(QWidget* parent) :
 	sp.setHorizontalPolicy(QSizePolicy::Expanding);
 
 	pageLayout->addWidget(gb, 1,0,13,4);
-	
+
 	gbGameModes = new QGroupBox(QGroupBox::tr("Game Modifiers"), gb);
 	gbBasicSettings = new QGroupBox(QGroupBox::tr("Basic Settings"), gb);
 
@@ -886,13 +979,13 @@ PageScheme::PageScheme(QWidget* parent) :
 	gbBasicSettings->setSizePolicy(sp);
 	gl->addWidget(gbGameModes,0,0,1,3,Qt::AlignTop);
 	gl->addWidget(gbBasicSettings,0,3,1,3,Qt::AlignTop);
-	
+
 	QGridLayout * glGMLayout = new QGridLayout(gbGameModes);
 	QGridLayout * glBSLayout = new QGridLayout(gbBasicSettings);
 	gbGameModes->setLayout(glGMLayout);
 	gbBasicSettings->setLayout(glBSLayout);
 	// Left
-	
+
 	TBW_mode_Forts = new ToggleButtonWidget(gbGameModes, ":/res/btnForts.png");
 	TBW_mode_Forts->setText(ToggleButtonWidget::tr("Fort Mode"));
     TBW_mode_Forts->setToolTip(tr("Defend your fort and destroy the opponents, two team colours max!"));
@@ -950,7 +1043,7 @@ PageScheme::PageScheme(QWidget* parent) :
 
 	// Right
 	QLabel * l;
-	
+
 	l = new QLabel(gbBasicSettings);
 	l->setText(QLabel::tr("Damage Modifier"));
 	l->setWordWrap(true);
@@ -959,7 +1052,7 @@ PageScheme::PageScheme(QWidget* parent) :
 	l->setFixedSize(32,32);
 	l->setPixmap(QPixmap(":/res/iconDamage.png"));
 	glBSLayout->addWidget(l,0,1,1,1);
-	
+
 	SB_DamageModifier = new QSpinBox(gbBasicSettings);
 	SB_DamageModifier->setRange(10, 300);
 	SB_DamageModifier->setValue(100);
@@ -974,13 +1067,13 @@ PageScheme::PageScheme(QWidget* parent) :
 	l->setFixedSize(32,32);
 	l->setPixmap(QPixmap(":/res/iconTime.png"));
 	glBSLayout->addWidget(l,1,1,1,1);
-	
+
 	SB_TurnTime = new QSpinBox(gbBasicSettings);
 	SB_TurnTime->setRange(1, 99);
 	SB_TurnTime->setValue(45);
 	SB_TurnTime->setSingleStep(15);
 	glBSLayout->addWidget(SB_TurnTime,1,2,1,1);
-	
+
 	l = new QLabel(gbBasicSettings);
 	l->setText(QLabel::tr("Initial Health"));
 	l->setWordWrap(true);
@@ -989,13 +1082,13 @@ PageScheme::PageScheme(QWidget* parent) :
 	l->setFixedSize(32,32);
 	l->setPixmap(QPixmap(":/res/iconHealth.png"));
 	glBSLayout->addWidget(l,2,1,1,1);
-	
+
 	SB_InitHealth = new QSpinBox(gbBasicSettings);
 	SB_InitHealth->setRange(50, 200);
 	SB_InitHealth->setValue(100);
 	SB_InitHealth->setSingleStep(25);
 	glBSLayout->addWidget(SB_InitHealth,2,2,1,1);
-	
+
 	l = new QLabel(gbBasicSettings);
 	l->setText(QLabel::tr("Sudden Death Timeout"));
 	l->setWordWrap(true);
@@ -1010,7 +1103,7 @@ PageScheme::PageScheme(QWidget* parent) :
 	SB_SuddenDeath->setValue(15);
 	SB_SuddenDeath->setSingleStep(3);
 	glBSLayout->addWidget(SB_SuddenDeath,3,2,1,1);
-	
+
 	l = new QLabel(gbBasicSettings);
 	l->setText(QLabel::tr("Crate Drops"));
 	l->setWordWrap(true);
@@ -1059,7 +1152,7 @@ PageScheme::PageScheme(QWidget* parent) :
 	l->setText(QLabel::tr("Scheme Name:"));
 
 	LE_name = new QLineEdit(this);
-	
+
 	gl->addWidget(LE_name,14,1,1,5);
 	gl->addWidget(l,14,0,1,1);
 
@@ -1082,7 +1175,7 @@ void PageScheme::setModel(QAbstractItemModel * model)
 {
 	mapper->setModel(model);
 	selectScheme->setModel(model);
-	
+
 	mapper->addMapping(LE_name, 0);
 	mapper->addMapping(TBW_mode_Forts->button(), 1);
 	mapper->addMapping(TBW_teamsDivide->button(), 2);
@@ -1141,7 +1234,9 @@ PageAdmin::PageAdmin(QWidget* parent) :
 	pageLayout->addWidget(leServerMessage, 0, 1);
 
 	pbSetSM = addButton(tr("Set message"), pageLayout, 0, 2);
-	BtnBack = addButton(":/res/Exit.png", pageLayout, 1, 0, true);
+	pbClearAccountsCache = addButton(tr("Clear Accounts Cache"), pageLayout, 1, 0);
+
+	BtnBack = addButton(":/res/Exit.png", pageLayout, 2, 0, true);
 
 	connect(pbSetSM, SIGNAL(clicked()), this, SLOT(smChanged()));
 }
