@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternSignatures #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 module NetRoutines where
 
 import Network
@@ -7,7 +7,7 @@ import System.IO
 import Control.Concurrent
 import Control.Concurrent.Chan
 import Control.Concurrent.STM
-import Control.Exception
+import qualified Control.Exception as Exception
 import Data.Time
 -----------------------------
 import CoreTypes
@@ -16,8 +16,8 @@ import Utils
 
 acceptLoop :: Socket -> Chan CoreMessage -> Int -> IO ()
 acceptLoop servSock coreChan clientCounter = do
-	Control.Exception.handle
-		(\(_ :: Exception) -> putStrLn "exception on connect") $
+	Exception.handle
+		(\(_ :: Exception.IOException) -> putStrLn "exception on connect") $
 		do
 		(socket, sockAddr) <- Network.Socket.accept servSock
 
@@ -45,6 +45,8 @@ acceptLoop servSock coreChan clientCounter = do
 					False
 					False
 					False
+					undefined
+					undefined
 					)
 
 		writeChan coreChan $ Accept newClient
